﻿using System;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using System.Windows.Navigation;
using Windows.Phone.Media.Capture;
using Microsoft.Devices;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Info;
using Size = Windows.Foundation.Size;

namespace LensApp
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        private PhotoCaptureDevice camera;
        private CameraSensorLocation currentCameraLocation;
        private bool isFlashEnabled;

        public MainPage()
        {
            InitializeComponent();
        }

        protected override async void OnNavigatedTo(NavigationEventArgs e)
        {
            Size resolution;
            string deviceName = DeviceStatus.DeviceName;
            //if (deviceName.Contains("RM-875") || deviceName.Contains("RM-876") || deviceName.Contains("RM-877"))
            //{
                resolution = new Size(7712, 4352);
            //}
            //else
            //{
                resolution = PhotoCaptureDevice.GetAvailableCaptureResolutions(CameraSensorLocation.Back).First();
            //}

            currentCameraLocation = CameraSensorLocation.Back;
            camera = await PhotoCaptureDevice.OpenAsync(CameraSensorLocation.Back, resolution);
            previewTransform.Rotation = camera.SensorRotationInDegrees;
            ApplyOrientation();
            isFlashEnabled = true;
            FlashStatus.Text = "Flash on";

            video.SetSource(camera);
        }

        protected override void OnNavigatingFrom(NavigatingCancelEventArgs e)
        {
            camera.Dispose();
        }

        private async void OnTakePictureClicked(object sender, GestureEventArgs gestureEventArgs)
        {
            CameraCaptureSequence cameraCaptureSequence = camera.CreateCaptureSequence(1);

            MemoryStream stream = new MemoryStream();
            cameraCaptureSequence.Frames[0].CaptureStream = stream.AsOutputStream();

            await camera.PrepareCaptureSequenceAsync(cameraCaptureSequence);
            await camera.FocusAsync();
            await cameraCaptureSequence.StartCaptureAsync();

            stream.Seek(0, SeekOrigin.Begin);

            (Application.Current as App).CapturedImage = stream;

            NavigationService.Navigate(new Uri("/Preview.xaml", UriKind.Relative));
        }

        protected override void OnOrientationChanged(OrientationChangedEventArgs e)
        {
            Orientation = e.Orientation;
            ApplyOrientation();
        }

        private void ApplyOrientation()
        {
            int rotation = 0;

            if (currentCameraLocation == CameraSensorLocation.Back)
            {
                switch (Orientation)
                {
                    case PageOrientation.LandscapeLeft:
                        {
                            rotation = 0;
                        } break;
                    case PageOrientation.PortraitUp:
                        {
                            rotation = 90;
                        } break;
                    case PageOrientation.LandscapeRight:
                        {
                            rotation = 180;
                        } break;
                    case PageOrientation.PortraitDown:
                        {
                            rotation = 270;
                        } break;
                }
            }
            else if (currentCameraLocation == CameraSensorLocation.Front)
            {
                switch (Orientation)
                {
                    case PageOrientation.LandscapeLeft:
                        {
                            rotation = 0;
                        } break;
                    case PageOrientation.PortraitUp:
                        {
                            rotation = 270;
                        } break;
                    case PageOrientation.LandscapeRight:
                        {
                            rotation = 180;
                        } break;
                    case PageOrientation.PortraitDown:
                        {
                            rotation = 90;
                        } break;
                }
            }

            if (camera != null)
            {
                camera.SetProperty(KnownCameraGeneralProperties.EncodeWithOrientation, rotation);
            }
        }

        private void OnChangeFlashStatus(object sender, EventArgs e)
        {
            if (isFlashEnabled)
            {
                camera.SetProperty(KnownCameraPhotoProperties.FlashMode, FlashState.Off);
                FlashStatus.Text = "Flash off";
                isFlashEnabled = false;
            }
            else
            {
                camera.SetProperty(KnownCameraPhotoProperties.FlashMode, FlashState.On);
                FlashStatus.Text = "Flash on";
                isFlashEnabled = true;
            }
        }
    }
}