﻿using System;
using System.IO;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Windows.Storage.Streams;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework.Media;
using Nokia.Graphics.Imaging;

namespace LensApp
{
    public partial class Preview : PhoneApplicationPage
    {
        private Stream capturedImage;
        private Stream currentImage;

        public Preview()
        {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (NavigationContext.QueryString.ContainsKey("token"))
            {
                string token = NavigationContext.QueryString["token"];
                MediaLibrary library = new MediaLibrary();
                Picture picture = library.GetPictureFromToken(token);
                capturedImage = picture.GetImage();
            }
            else
            {
                capturedImage = (Application.Current as App).CapturedImage;
            }

            currentImage = capturedImage;

            BitmapImage image = new BitmapImage();
            image.SetSource(capturedImage);
            CapturedImage.Source = image;
        }

        private async void OnBlackWhiteClicked(object sender, EventArgs e)
        {
            capturedImage.Seek(0, SeekOrigin.Begin);
            using (EditingSession session = await EditingSessionFactory.CreateEditingSessionAsync(capturedImage))
            {
                session.AddFilter(FilterFactory.CreateGrayscaleFilter());
                await session.RenderToImageAsync(CapturedImage);
                IBuffer buffer = await session.RenderToJpegAsync();
                currentImage = buffer.AsStream();
            }
        }

        private async void OnSeppiaClicked(object sender, EventArgs e)
        {
            capturedImage.Seek(0, SeekOrigin.Begin);
            using (EditingSession session = await EditingSessionFactory.CreateEditingSessionAsync(capturedImage))
            {
                session.AddFilter(FilterFactory.CreateSepiaFilter());
                await session.RenderToImageAsync(CapturedImage);
                IBuffer buffer = await session.RenderToJpegAsync();
                currentImage = buffer.AsStream();
            }
        }

        private async void OnSketchClicked(object sender, EventArgs e)
        {
            capturedImage.Seek(0, SeekOrigin.Begin);
            using (EditingSession session = await EditingSessionFactory.CreateEditingSessionAsync(capturedImage))
            {
                session.AddFilter(FilterFactory.CreateSketchFilter(SketchMode.Color));
                await session.RenderToImageAsync(CapturedImage);
                IBuffer buffer = await session.RenderToJpegAsync();
                currentImage = buffer.AsStream();
            }
        }

        private void OnSaveClicked(object sender, EventArgs e)
        {
            currentImage.Seek(0, SeekOrigin.Begin);
            MediaLibrary library = new MediaLibrary();
            library.SavePictureToCameraRoll(Guid.NewGuid().ToString(), currentImage);
            MessageBox.Show("Picture saved");
        }
    }
}